@@ -41,6 +41,8 @@
 #include <gtksourceview/gtksourcestylescheme.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
 #include <gtksourceview/gtksourceundomanager.h>
+#include <gtksourceview/gtksourceutils.h>
 #include <gtksourceview/gtksourceview.h>
+#include <gtksourceview/gtksourceview-typebuiltins.h>
 
 #endif /* __GTK_SOURCE_H__ */
