@@ -20,10 +20,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 /**
  * SECTION:utils
  * @title: GtkSourceUtils
@@ -39,12 +35,9 @@
  * gtk_source_utils_unescape_search_text:
  * @text: the text to unescape.
  *
- * Use this function before gtk_source_search_settings_set_search_text(), to
- * unescape the following sequences of characters: `\n`, `\r`, `\t` and `\\`.
- * The purpose is to easily write those characters in a search entry.
- *
- * Note that unescaping the search text is not needed for regular expression
- * searches.
+ * Use this function before gtk_source_buffer_set_search_text(), to unescape
+ * certain sequences of characters: \n, \r, \t and \\. The purpose is to easily
+ * write those characters in a search entry.
  *
  * See also: gtk_source_utils_escape_search_text().
  *
@@ -131,23 +124,11 @@ gtk_source_utils_unescape_search_text (const gchar *text)
  * gtk_source_utils_escape_search_text:
  * @text: the text to escape.
  *
- * Use this function to escape the following characters: `\n`, `\r`, `\t` and `\`.
- *
- * For a regular expression search, use g_regex_escape_string() instead.
- *
- * One possible use case is to take the #GtkTextBuffer's selection and put it in a
- * search entry. The selection can contain tabulations, newlines, etc. So it's
- * better to escape those special characters to better fit in the search entry.
+ * Use this function after gtk_source_buffer_get_search_text(), to escape
+ * certain characters: \n, \r, \t and \.
  *
  * See also: gtk_source_utils_unescape_search_text().
  *
- * <warning>
- * Warning: the escape and unescape functions are not reciprocal! For example,
- * escape (unescape (\)) = \\. So avoid cycles such as: search entry -> unescape
- * -> search settings -> escape -> search entry. The original search entry text
- * may be modified.
- * </warning>
- *
  * Returns: the escaped @text.
  * Since: 3.10
  */
@@ -166,6 +147,14 @@ gtk_source_utils_escape_search_text (const gchar* text)
 
 	length = strlen (text);
 
+	/* no escape when typing.
+	 * The short circuit works only for ascii, but we only
+	 * care about not escaping a single '\' */
+	if (length == 1)
+	{
+		return g_strdup (text);
+	}
+
 	str = g_string_new ("");
 
 	p = text;
@@ -173,7 +162,8 @@ gtk_source_utils_escape_search_text (const gchar* text)
 
 	while (p != end)
 	{
-		const gchar *next = g_utf8_next_char (p);
+		const gchar *next;
+		next = g_utf8_next_char (p);
 
 		switch (*p)
 		{
